/*************************************************************************
 *
 *  $RCSfile: collator_simple.cxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: er $ $Date: 2002/03/26 17:02:11 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <collator_simple.hxx>
#include <tools/isolang.hxx>
#include <com/sun/star/i18n/CollatorOptions.hpp>

using namespace ::com::sun::star;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::uno;
using namespace ::rtl;

namespace com { namespace sun { namespace star { namespace i18n {

Collator_Simple::Collator_Simple () : mpInternational (NULL), mnCompFlags (0)
{
	implementationName = "com.sun.star.i18n.Collator_Simple";
}

Collator_Simple::~Collator_Simple ()
{
	if (mpInternational != NULL)
	    delete mpInternational;
}

sal_Int32 SAL_CALL
Collator_Simple::compareSubstring ( const OUString& str1, sal_Int32 off1, sal_Int32 len1, 
	const OUString& str2, sal_Int32 off2, sal_Int32 len2) throw (RuntimeException)
{
	return compareString(str1.copy(off1, len1), str2.copy(off2, len2));
}

sal_Int32 SAL_CALL
Collator_Simple::compareString(const OUString& str1, const OUString& str2) throw (RuntimeException)
{
	if (mpInternational == NULL)
	    throw RuntimeException();

	// let the International class do the work
	switch(mpInternational->Compare(str1, str2, mnCompFlags)) {
	    case COMPARE_LESS: 
		return -1;
	    case COMPARE_EQUAL: 
		return 0;
	    default: // COMPARE_GREATER)
		return 1;
	}
}

sal_Int32 SAL_CALL 
Collator_Simple::loadCollatorAlgorithm(const OUString& impl, const lang::Locale& rLocale,
	sal_Int32 collatorOptions) throw(RuntimeException)
{
	// convert the Locale to Language
	LanguageType eLang = ConvertIsoNamesToLanguage(rLocale.Language, rLocale.Country);
	if (eLang == LANGUAGE_DONTKNOW)
	    eLang = LANGUAGE_ENGLISH_US;

	// make sure the International member is sane and fits the language
	if (mpInternational == NULL)
	    mpInternational = new International(eLang);
	else if (mpInternational->GetLanguage() != eLang) {
	    delete mpInternational;
	    mpInternational = new International(eLang);
	}

	if (collatorOptions & i18n::CollatorOptions::CollatorOptions_IGNORE_CASE)
	    mnCompFlags = INTN_COMPARE_IGNORECASE;
	else
	    mnCompFlags = 0;

    return 0;
}

} } } }
