/*************************************************************************
#*
#*    $RCSfile: log.hxx,v $
#*
#*    class Log
#*
#*
#*    Implementation    log.cxx
#*
#*    Description       class for logging program progress, state and errors
#*                      to files or stdout
#*
#*    Creation date     sz  09/27/2000
#*    last change       $Author: vg $ $Date: 2003/05/19 13:12:24 $
#*    $Revision: 1.4 $
#*
#*    Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
#*
#*************************************************************************

      Source Code Control System - Header

      $Header: /cvs/tools/testshl2/inc/cppunit/result/log.hxx,v 1.4 2003/05/19 13:12:24 vg Exp $

      Source Code Control System - Update

*************************************************************************/
#ifndef __QADEV_REGSCAN_LOG_HXX__
#define __QADEV_REGSCAN_LOG_HXX__

#include <time.h>

#ifndef _RTL_USTRING_HXX_
#include	<rtl/ustring.hxx>
#endif

#ifndef _RTL_STRBUF_HXX_
#include	<rtl/strbuf.hxx>
#endif

#ifndef _SAL_TYPES_H_
#include	<sal/types.h>
#endif

#ifndef _OSL_THREAD_H_
#include	<osl/thread.h>
#endif

#ifndef _OSL_FILE_HXX_
#include	<osl/file.hxx>
#endif

#include <cppunit/nocopy.hxx>

//!io #include <iostream>
#include <vector>

using namespace std;


/**
 * Log derives the interface of the ::osl::File class ( is-a relation ).
 * Its members (has-a relation) are the (full qualified)name of the log
 * and an OStringBuffer which represents the content of the logfile.
 * It provides the functionality of easy-to-use open and write logs
 */
//: Log
class Log : NOCOPY {

    ::osl::File*            m_logfile;          // fileobject
    rtl::OUString         m_logurl;           // url of log
	rtl::OStringBuffer	m_buf;              // content of log

    Log();

public:

	//> c'tor
	/**
	 * constructors argument is a full qualified UNC path
	 * @param OUString logfile ( full qualified UNC path )
	 */
    Log( const rtl::OUString& logURL )
            : m_logfile( new ::osl::File( logURL ))
			, m_logurl(logURL)
    {} ///< c'tor

    //> d'tor
    virtual ~Log()
        {
            m_logfile->close();
            delete( m_logfile );
        } ///< d'tor

	//> inline methods
	// returns a reference to name instance
    inline rtl::OUString getLogURL() { return m_logurl; }
    inline rtl::OString getName() { return rtl::OUStringToOString(
                                    m_logurl, RTL_TEXTENCODING_ASCII_US  ); }
	///< inline methods

	// open logfile for overwrite (default) or append
	::osl::FileBase::RC open( sal_Bool append = sal_False );
	::osl::FileBase::RC close() { return m_logfile->close(); }


	// write methods without (default) or with echo on display
	::osl::FileBase::RC write( const sal_Char* buf, sal_Bool v = sal_False );
	::osl::FileBase::RC write( const rtl::OString& buf,
													sal_Bool v = sal_False );
	//! ::osl::FileBase::RC write( rtl::OStringBuffer& buf,
	//! 												sal_Bool v = sal_False );
	::osl::FileBase::RC write( const rtl::OUString& buf,
							rtl_TextEncoding enc = RTL_TEXTENCODING_ASCII_US,
												 sal_Bool v = sal_False   );

}; ///:~ Log

#endif
