/*************************************************************************
 *
 *  $RCSfile: optionhelper.hxx,v $
 *
 *  $Revision: 1.6 $
 *
 *  last change: $Author: vg $ $Date: 2003/05/19 13:12:37 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef optionhelper_hxx
#define optionhelper_hxx

#include <string>

#include <sal/types.h>

#include "cppunit/nocopy.hxx"
#include "getopt.hxx"
#include "cppunit/joblist.hxx"

// -----------------------------------------------------------------------------

typedef std::vector<rtl::OString> OStringList;

//!? Better: OptionHelper
class OptionHelper : NOCOPY
{
    GetOpt & m_aOption;
    JobList     m_aJobOnlyList;
    OStringList m_aJobFilter;
    
    std::string m_sProjectId;
    std::string m_sBuildId;
    
    std::string getProjectId() const;
    std::string getBuildId() const;
    std::string createDateTag(std::string const& _sProjectId, std::string const& _sBuildId);
    
    void handleJobs();
public:
    OptionHelper(GetOpt & _aOption)
            :m_aOption(_aOption)
        {
            if (m_aOption.hasOpt("-projectid"))
                m_sProjectId = m_aOption.getOpt("-projectid");
            
            if (m_aOption.hasOpt("-buildid"))
                m_sBuildId = m_aOption.getOpt("-buildid");

			handleJobs();
        }
    
    static std::string integerToAscii(sal_Int32 nValue);

    std::string createDateTag();
    bool showErrors();
    bool showTests();

    JobList getJobOnlyList() {return m_aJobOnlyList;}
    
    bool isAllowedToExecute(std::string const& _sNode, std::string const& _sName);

    bool isOnlyShowJobs() {return m_aOption.hasOpt("-onlyshowjobs") == sal_True ? true : false;}
	GetOpt& getOptions() {return m_aOption;}
	bool isVerbose() {return m_aOption.hasOpt("-verbose") == sal_True ? true : false;} 
};

// -----------------------------------------------------------------------------


#endif


